@@ -1,10 +1,12 @@
 """Base class for undirected hypergraphs."""
+from collections import defaultdict
 from collections.abc import Hashable, Iterable
-from copy import deepcopy
+from copy import copy, deepcopy
 from itertools import count
 from warnings import warn
 
 from ..exception import IDNotFound, XGIError
+from ..utils.utilities import update_uid_counter
 from .reportviews import EdgeView, NodeView
 
 __all__ = ["Hypergraph"]
@@ -98,6 +100,51 @@ class Hypergraph:
     _hyperedge_attr_dict_factory = IDDict
     _hypergraph_attr_dict_factory = dict
 
+    def __getstate__(self):
+        """Function that allows pickling.
+
+        Returns
+        -------
+        dict
+            The keys label the hyeprgraph dict and the values
+            are dictionarys from the Hypergraph class.
+
+        Notes
+        -----
+        This allows the python multiprocessing module to be used.
+
+        """
+        return {
+            "_edge_uid": self._edge_uid,
+            "_hypergraph": self._hypergraph,
+            "_node": self._node,
+            "_node_attr": self._node_attr,
+            "_edge": self._edge,
+            "_edge_attr": self._edge_attr,
+        }
+
+    def __setstate__(self, state):
+        """Function that allows unpickling of a hypergraph.
+
+        Parameters
+        ----------
+        state
+            The keys access the dictionary names the values are the
+            dictionarys themselves from the Hypergraph class.
+
+        Notes
+        -----
+        This allows the python multiprocessing module to be used.
+        """
+        self._edge_uid = state["_edge_uid"]
+        self._hypergraph = state["_hypergraph"]
+        self._node = state["_node"]
+        self._node_attr = state["_node_attr"]
+        self._edge = state["_edge"]
+        self._edge_attr = state["_edge_attr"]
+        self._nodeview = NodeView(self)
+        self._edgeview = EdgeView(self)
+
     def __init__(self, incoming_data=None, **attr):
         self._edge_uid = count()
         self._hypergraph = self._hypergraph_attr_dict_factory()
@@ -483,7 +530,12 @@ class Hypergraph:
         if not members:
             raise XGIError("Cannot add an empty edge")
 
+        if id in self._edge.keys():  # check that uid is not present yet
+            warn(f"uid {id} already exists, cannot add edge {members}")
+            return
+
         uid = next(self._edge_uid) if not id else id
+
         self._edge[uid] = set()
         for node in members:
             if node not in self._node:
@@ -495,6 +547,9 @@ class Hypergraph:
         self._edge_attr[uid] = self._hyperedge_attr_dict_factory()
         self._edge_attr[uid].update(attr)
 
+        if id:  # set self._edge_uid correctly
+            update_uid_counter(self, id)
+
     def add_edges_from(self, ebunch_to_add, **attr):
         """Add multiple edges with optional attributes.
 
@@ -502,20 +557,21 @@ class Hypergraph:
         ----------
         ebunch_to_add : Iterable
 
-            An iterable of edges.  This may be a dict of the form `{edge_id:
-            edge_members}`, or it may be an iterable of iterables, where each element
-            contains the members of the edge specified as valid node IDs.
+            An iterable of edges.  This may be an iterable of iterables (Format 1),
+            where each element contains the members of the edge specified as valid node IDs.
             Alternatively, each element could also be a tuple in any of the following
             formats:
 
-            * Format 1: 2-tuple (members, edge_id), or
-            * Format 2: 2-tuple (members, attr), or
-            * Format 3: 3-tuple (members, edge_id, attr),
+            * Format 2: 2-tuple (members, edge_id), or
+            * Format 3: 2-tuple (members, attr), or
+            * Format 4: 3-tuple (members, edge_id, attr),
 
             where `members` is an iterable of node IDs, `edge_id` is a hashable to use
-            as edge ID, and `attr` is a dict of attributes. The first and second formats
-            are unambiguous because `attr` dicts are not hashable, while `id`s must be.
-            In Formats 1-3, each element of `ebunch_to_add` must have the same length,
+            as edge ID, and `attr` is a dict of attributes. Finally, `ebunch_to_add`
+            may be a dict of the form `{edge_id: edge_members}` (Format 5).
+
+            Formats 2 and 3 are unambiguous because `attr` dicts are not hashable, while `id`s must be.
+            In Formats 2-4, each element of `ebunch_to_add` must have the same length,
             i.e. you cannot mix different formats.  The iterables containing edge
             members cannot be strings.
 
@@ -594,17 +650,23 @@ class Hypergraph:
         """
         # format 5 is the easiest one
         if isinstance(ebunch_to_add, dict):
-            for uid, members in ebunch_to_add.items():
+            for id, members in ebunch_to_add.items():
+                if id in self._edge.keys():  # check that uid is not present yet
+                    warn(f"uid {id} already exists, cannot add edge {members}.")
+                    continue
                 try:
-                    self._edge[uid] = set(members)
+                    self._edge[id] = set(members)
                 except TypeError as e:
                     raise XGIError("Invalid ebunch format") from e
                 for n in members:
                     if n not in self._node:
                         self._node[n] = set()
                         self._node_attr[n] = self._node_attr_dict_factory()
-                    self._node[n].add(uid)
-                self._edge_attr[uid] = self._hyperedge_attr_dict_factory()
+                    self._node[n].add(id)
+                self._edge_attr[id] = self._hyperedge_attr_dict_factory()
+
+                update_uid_counter(self, id)
+
             return
 
         # in formats 1-4 we only know that ebunch_to_add is an iterable, so we iterate
@@ -641,32 +703,39 @@ class Hypergraph:
         e = first_edge
         while True:
             if format1:
-                members, uid, eattr = e, next(self._edge_uid), {}
+                members, id, eattr = e, next(self._edge_uid), {}
             elif format2:
-                members, uid, eattr = e[0], e[1], {}
+                members, id, eattr = e[0], e[1], {}
             elif format3:
-                members, uid, eattr = e[0], next(self._edge_uid), e[1]
+                members, id, eattr = e[0], next(self._edge_uid), e[1]
             elif format4:
-                members, uid, eattr = e[0], e[1], e[2]
+                members, id, eattr = e[0], e[1], e[2]
 
-            try:
-                self._edge[uid] = set(members)
-            except TypeError as e:
-                raise XGIError("Invalid ebunch format") from e
+            if id in self._edge.keys():  # check that uid is not present yet
+                warn(f"uid {id} already exists, cannot add edge {members}.")
+            else:
 
-            for n in members:
-                if n not in self._node:
-                    self._node[n] = set()
-                    self._node_attr[n] = self._node_attr_dict_factory()
-                self._node[n].add(uid)
+                try:
+                    self._edge[id] = set(members)
+                except TypeError as e:
+                    raise XGIError("Invalid ebunch format") from e
 
-            self._edge_attr[uid] = self._hyperedge_attr_dict_factory()
-            self._edge_attr[uid].update(attr)
-            self._edge_attr[uid].update(eattr)
+                for n in members:
+                    if n not in self._node:
+                        self._node[n] = set()
+                        self._node_attr[n] = self._node_attr_dict_factory()
+                    self._node[n].add(id)
+
+                self._edge_attr[id] = self._hyperedge_attr_dict_factory()
+                self._edge_attr[id].update(attr)
+                self._edge_attr[id].update(eattr)
 
             try:
                 e = next(new_edges)
             except StopIteration:
+
+                if format2 or format4:
+                    update_uid_counter(self, id)
                 break
 
     def add_weighted_edges_from(self, ebunch, weight="weight", **attr):
@@ -941,6 +1010,169 @@ class Hypergraph:
         self._edge.clear()
         self._edge_attr.clear()
 
+    def merge_duplicate_edges(
+        self, rename="first", merge_rule="first", multiplicity=None
+    ):
+        """Merges edges which have the same members.
+
+        Parameters
+        ----------
+        rename : str, optional
+            Either "first", "tuple", or "new", by default "first"
+            If "first", the new edge ID is the first of the sorted
+            duplicate edge IDs. If "tuple", the new edge ID is a
+            tuple of the sorted duplicate edge IDs. If "new", a
+            new ID will be selected automatically.
+        merge_rule : str, optional
+            Either "first" or "union", by default "first"
+            If "first", takes the attributes of the first duplicate.
+            If "union", takes the set of attributes of all the duplicates.
+        multiplicity : str, optional
+            The attribute in which to store the multiplicity of the hyperedge,
+            by default None
+
+        Raises
+        ------
+        XGIError
+            If invalid rename or merge_rule specified.
+
+        Warns
+        -----
+        If the user chooses merge_rule="union". Tells the
+        user that they can no longer draw based on this stat.
+
+        Examples
+        --------
+
+        >>> import xgi
+        >>> edges = [{1, 2}, {1, 2}, {1, 2}, {3, 4, 5}, {3, 4, 5}]
+        >>> edge_attrs = dict()
+        >>> edge_attrs[0] = {"color": "blue"}
+        >>> edge_attrs[1] = {"color": "red", "weight": 2}
+        >>> edge_attrs[2] = {"color": "yellow"}
+        >>> edge_attrs[3] = {"color": "purple"}
+        >>> edge_attrs[4] = {"color": "purple", "name": "test"}
+        >>> H = xgi.Hypergraph(edges)
+        >>> xgi.set_edge_attributes(H, edge_attrs)
+        >>> H.edges
+        EdgeView((0, 1, 2, 3, 4))
+
+        There are several ways to rename the duplicate edges after merging:
+
+        1. The merged edge ID is the first duplicate edge ID.
+
+        >>> H1 = H.copy()
+        >>> H1.merge_duplicate_edges()
+        >>> H1.edges
+        EdgeView((0, 3))
+
+        2. The merged edge ID is a tuple of all the duplicate edge IDs.
+
+        >>> H2 = H.copy()
+        >>> H2.merge_duplicate_edges(rename="tuple")
+        >>> H2.edges
+        EdgeView(((0, 1, 2), (3, 4)))
+
+        3. The merged edge ID is assigned a new edge ID.
+
+        >>> H3 = H.copy()
+        >>> H3.merge_duplicate_edges(rename="new")
+        >>> H3.edges
+        EdgeView((5, 6))
+
+        We can also specify how we would like to combine the attributes
+        of the merged edges:
+
+        1. The attributes are the attributes of the first merged edge.
+
+        >>> H4 = H.copy()
+        >>> H4.merge_duplicate_edges()
+        >>> H4.edges[0]
+        {'color': 'blue'}
+
+        2. The attributes are the union of every attribute that each merged
+        edge has. If a duplicate edge doesn't have that attribute, it is set
+        to None.
+
+        >>> H5 = H.copy()
+        >>> H5.merge_duplicate_edges(merge_rule="union")
+        >>> H5.edges[0] == {'color': {'blue', 'red', 'yellow'}, 'weight':{2, None}}
+        True
+
+        3. We can also set the attributes to the intersection, i.e.,
+        if a particular attribute is the same across the duplicate
+        edges, we use this attribute, otherwise, we set it to None.
+
+        >>> H6 = H.copy()
+        >>> H6.merge_duplicate_edges(merge_rule="intersection")
+        >>> H6.edges[0] == {'color': None, 'weight': None}
+        True
+        >>> H6.edges[3] == {'color': 'purple', 'name': None}
+        True
+
+        We can also choose to store the multiplicity of the edge
+        as an attribute. The user simply provides the string of
+        the attribute which stores it. Note that this will not prevent
+        other attributes from being over written (e.g., weight), so
+        be careful that the attribute is not already in use.
+
+        >>> H7 = H.copy()
+        >>> H7.merge_duplicate_edges(multiplicity="mult")
+        >>> H7.edges[0]['mult'] == 3
+        True
+        """
+        dups = []
+        hashes = defaultdict(list)
+        for idx, members in self._edge.items():
+            hashes[frozenset(members)].append(idx)
+
+        new_edges = list()
+        for members, dup_ids in hashes.items():
+            if len(dup_ids) > 1:
+                dups.extend(dup_ids)
+
+                if rename == "first":
+                    new_id = sorted(dup_ids)[0]
+                elif rename == "tuple":
+                    new_id = tuple(sorted(dup_ids))
+                elif rename == "new":
+                    new_id = next(self._edge_uid)
+                else:
+                    raise XGIError("Invalid ID renaming scheme!")
+
+                if merge_rule == "first":
+                    id = min(dup_ids)
+                    new_attrs = deepcopy(self._edge_attr[id])
+                elif merge_rule == "union":
+                    attrs = {field for id in dup_ids for field in self._edge_attr[id]}
+                    new_attrs = {
+                        attr: {self._edge_attr[id].get(attr) for id in dup_ids}
+                        for attr in attrs
+                    }
+                elif merge_rule == "intersection":
+                    attrs = {field for id in dup_ids for field in self._edge_attr[id]}
+                    set_attrs = {
+                        attr: {self._edge_attr[id].get(attr) for id in dup_ids}
+                        for attr in attrs
+                    }
+                    new_attrs = {
+                        attr: (None if len(val) != 1 else next(iter(val)))
+                        for attr, val in set_attrs.items()
+                    }
+                else:
+                    raise XGIError("Invalid merge rule!")
+
+                if multiplicity is not None:
+                    new_attrs[multiplicity] = len(dup_ids)
+                new_edges.append((members, new_id, new_attrs))
+        self.remove_edges_from(dups)
+        self.add_edges_from(new_edges)
+
+        if merge_rule == "union":
+            warn(
+                "You will not be able to color/draw by merged attributes with xgi.draw()!"
+            )
+
     def copy(self):
         """A deep copy of the hypergraph.
 
@@ -951,51 +1183,20 @@ class Hypergraph:
         H : Hypergraph
             A copy of the hypergraph.
 
-        Notes
-        -----
-
-        There is no guarantee that performing similar operations on a hypergraph and its
-        copy after the copy is made will yield the same results.  For example,
-
-        >>> import xgi
-        >>> H = xgi.Hypergraph([[1, 2, 3], [4], [5, 6], [6, 7, 8]])
-        >>> H.add_edge([1, 3, 5], id=10)
-        >>> K = H.copy()
-        >>> H.add_edge([2, 4]); K.add_edge([2, 4]);
-        >>> list(H.edges) == list(K.edges)
-        False
-
-        The difference is the IDs assigned to new edges:
-
-        >>> H.edges
-        EdgeView((0, 1, 2, 3, 10, 4))
-        >>> K.edges
-        EdgeView((0, 1, 2, 3, 10, 11))
-
         """
-        copy = self.__class__()
+        cp = self.__class__()
         nn = self.nodes
-        copy.add_nodes_from((n, deepcopy(attr)) for n, attr in nn.items())
+        cp.add_nodes_from((n, deepcopy(attr)) for n, attr in nn.items())
         ee = self.edges
-        copy.add_edges_from(
+        cp.add_edges_from(
             (e, id, deepcopy(self.edges[id]))
             for id, e in ee.members(dtype=dict).items()
         )
-        copy._hypergraph = deepcopy(self._hypergraph)
+        cp._hypergraph = deepcopy(self._hypergraph)
 
-        # If we don't set the start of copy._edge_uid correctly, it will start at 0,
-        # which will overwrite any existing edges when calling add_edge().  First, we
-        # use the somewhat convoluted float(e).is_integer() instead of using
-        # isinstance(e, int) because there exist integer-like numeric types (such as
-        # np.int32) which fail the isinstance() check.
-        edges_with_int_id = [int(e) for e in self.edges if float(e).is_integer()]
+        cp._edge_uid = copy(self._edge_uid)
 
-        # Then, we set the start at one plus the maximum edge ID that is an integer,
-        # because count() only yields integer IDs.
-        start = max(edges_with_int_id) + 1 if edges_with_int_id else 0
-        copy._edge_uid = count(start=start)
-
-        return copy
+        return cp
 
     def dual(self):
         """The dual of the hypergraph.
@@ -1044,7 +1245,7 @@ class Hypergraph:
         """
         if in_place:
             if not multiedges:
-                self.remove_edges_from(self.edges.duplicates())
+                self.merge_duplicate_edges()
             if not singletons:
                 self.remove_edges_from(self.edges.singletons())
             if not isolates:
@@ -1069,7 +1270,7 @@ class Hypergraph:
             if not multiedges:
                 H.remove_edges_from(H.edges.duplicates())
             if not singletons:
-                H.remove_edges_from(H.edges.singletons())
+                H.merge_duplicate_edges()
             if not isolates:
                 H.remove_nodes_from(H.nodes.isolates())
             if relabel:
